%token DEC_OP
%token IF
%token LEFT_OP
%token SIZEOF
%token EQ_OP
%token WHILE
%token DO
%token RIGHT_OP
%token INC_OP
%token ENUM
%token Term_storage_class_specifier
%token OR_OP
%token Term_identifier
%token ELSE
%token SWITCH
%token PTR_OP
%token RETURN
%token CASE
%token ELIPSIS
%token Term_struct_or_union
%token FOR
%token NE_OP
%token AND_OP
%token LE_OP
%token GE_OP
%token DEFAULT

%%

file :  external_definition
|  file external_definition
;

jump_statement :  RETURN expr ';'
;

parameter_identifier_list :  identifier_list
|  identifier_list ',' ELIPSIS
;

and_expr :  equality_expr
|  and_expr '&' equality_expr
;

struct_or_union_specifier :  Term_struct_or_union '{' struct_declaration_list '}'
|  Term_struct_or_union Term_identifier
|  Term_struct_or_union Term_identifier '{' struct_declaration_list '}'
;

compound_statement :  '{' declaration_list '}'
|  '{' statement_list '}'
|  '{' declaration_list statement_list '}'
;

initializer_list :  initializer_list ',' initializer
|  initializer
;

struct_declarator :  declarator ':' constant_expr
|  ':' constant_expr
|  declarator
;

logical_or_expr :  logical_and_expr
|  logical_or_expr OR_OP logical_and_expr
;

equality_expr :  equality_expr EQ_OP relational_expr
|  equality_expr NE_OP relational_expr
|  relational_expr
;

function_definition :  declaration_specifiers declarator function_body
|  declarator function_body
;

declaration_specifiers :  type_specifier declaration_specifiers
|  Term_storage_class_specifier
|  type_specifier
|  Term_storage_class_specifier declaration_specifiers
;

assignment_operator :  ','
|  '='
;

logical_and_expr :  inclusive_or_expr
|  logical_and_expr AND_OP inclusive_or_expr
;

shift_expr :  shift_expr LEFT_OP additive_expr
|  shift_expr RIGHT_OP additive_expr
|  additive_expr
;

primary_expr :  '(' expr ')'
|  Term_identifier
;

struct_declaration :  type_specifier_list struct_declarator_list ';'
;

conditional_expr :  logical_or_expr
|  logical_or_expr '?' logical_or_expr ':' conditional_expr
;

iteration_statement :  FOR '(' ';' ';' expr ')' statement
|  DO statement WHILE '(' expr ')' ';'
|  FOR '(' expr ';' ';' expr ')' statement
|  WHILE '(' expr ')' statement
|  FOR '(' expr ';' expr ';' expr ')' statement
|  FOR '(' ';' expr ';' expr ')' statement
|  FOR '(' expr ';' ';' ')' statement
|  FOR '(' ';' ';' ')' statement
|  FOR '(' ';' expr ';' ')' statement
|  FOR '(' expr ';' expr ';' ')' statement
;

relational_expr :  relational_expr '>' shift_expr
|  relational_expr GE_OP shift_expr
|  relational_expr '<' shift_expr
|  relational_expr LE_OP shift_expr
|  shift_expr
;

additive_expr :  multiplicative_expr
|  additive_expr '-' multiplicative_expr
|  additive_expr '+' multiplicative_expr
;

expression_statement :  ';'
|  expr ';'
;

external_definition :  function_definition
|  declaration
;

unary_expr :  INC_OP unary_expr
|  DEC_OP unary_expr
|  SIZEOF unary_expr
|  unary_operator cast_expr
|  SIZEOF '(' type_name ')'
|  postfix_expr
;

abstract_declarator2 :  '[' constant_expr ']'
|  abstract_declarator2 '[' ']'
|  abstract_declarator2 '(' parameter_type_list ')'
|  abstract_declarator2 '(' ')'
|  abstract_declarator2 '[' constant_expr ']'
|  '(' parameter_type_list ')'
|  '(' abstract_declarator ')'
;

enumerator :  Term_identifier '=' constant_expr
|  Term_identifier
;

labeled_statement :  CASE constant_expr ':' statement
|  Term_identifier ':' statement
|  DEFAULT ':' statement
;

declaration_list :  declaration
|  declaration_list declaration
;

struct_declarator_list :  struct_declarator_list ',' struct_declarator
|  struct_declarator
;

enum_specifier :  ENUM Term_identifier '{' enumerator_list '}'
|  ENUM '{' enumerator_list '}'
|  ENUM Term_identifier
;

init_declarator :  declarator
|  declarator '=' initializer
;

postfix_expr :  postfix_expr '(' ')'
|  primary_expr
|  postfix_expr PTR_OP Term_identifier
|  postfix_expr DEC_OP
|  postfix_expr '[' expr ']'
|  postfix_expr '.' Term_identifier
|  postfix_expr '(' argument_expr_list ')'
|  postfix_expr INC_OP
;

multiplicative_expr :  multiplicative_expr '*' cast_expr
|  cast_expr
|  multiplicative_expr '%' cast_expr
|  multiplicative_expr '/' cast_expr
;

inclusive_or_expr :  exclusive_or_expr
|  inclusive_or_expr '|' exclusive_or_expr
;

pointer :  '*' type_specifier_list
|  '*' pointer
|  '*' type_specifier_list pointer
|  '*'
;

init_declarator_list :  init_declarator
|  init_declarator_list ',' init_declarator
;

selection_statement :  IF '(' expr ')' statement ELSE statement
|  SWITCH '(' expr ')' statement
;

declaration :  declaration_specifiers init_declarator_list ';'
|  declaration_specifiers ';'
;

type_specifier :  struct_or_union_specifier
|  enum_specifier
;

enumerator_list :  enumerator_list ',' enumerator
|  enumerator
;

cast_expr :  unary_expr
|  '(' type_name ')' cast_expr
;

expr :  expr ',' assignment_expr
|  assignment_expr
;

parameter_list :  parameter_declaration
|  parameter_list ',' parameter_declaration
;

unary_operator :  '*'
|  '+'
|  '-'
|  '&'
;

identifier_list :  Term_identifier
|  identifier_list ',' Term_identifier
;

declarator :  pointer declarator2
|  declarator2
;

exclusive_or_expr :  exclusive_or_expr '^' and_expr
|  and_expr
;

type_name :  type_specifier_list
|  type_specifier_list abstract_declarator
;

constant_expr :  conditional_expr
;

statement :  compound_statement
|  selection_statement
|  expression_statement
|  iteration_statement
|  jump_statement
|  labeled_statement
;

abstract_declarator :  pointer
|  pointer abstract_declarator2
|  abstract_declarator2
;

statement_list :  statement_list statement
|  statement
;

struct_declaration_list :  struct_declaration
|  struct_declaration_list struct_declaration
;

parameter_type_list :  parameter_list ',' ELIPSIS
|  parameter_list
;

function_body :  compound_statement
|  declaration_list compound_statement
;

assignment_expr :  conditional_expr
|  unary_expr assignment_operator assignment_expr
;

argument_expr_list :  assignment_expr
|  argument_expr_list ',' assignment_expr
;

type_specifier_list :  type_specifier
|  type_specifier_list type_specifier
;

parameter_declaration :  type_specifier_list declarator
|  type_name
;

initializer :  assignment_expr
|  '{' initializer_list '}'
|  '{' initializer_list ',' '}'
;

declarator2 :  declarator2 '(' parameter_identifier_list ')'
|  Term_identifier
|  '(' declarator ')'
|  declarator2 '[' ']'
|  declarator2 '(' parameter_type_list ')'
|  declarator2 '(' ')'
|  declarator2 '[' constant_expr ']'
;
