%token DOTDOT
%token IF
%token THEN
%token Term_mult_op
%token REPEAT
%token FUNCTION
%token OF
%token SBEGIN
%token Term_external_files
%token LABEL
%token Term_add_op
%token WHILE
%token STRING
%token RECORD
%token SFILE
%token Term_direction
%token DO
%token VAR
%token WITH
%token UNSIGNED_INT
%token CONST
%token SET
%token UNTIL
%token END
%token TYPE
%token ARRAY
%token ELSE
%token PACKED
%token PROCEDURE
%token NOT
%token CASE
%token BECOMES
%token IDENTIFIER
%token FOR
%token PROGRAM

%%

program :  PROGRAM newident Term_external_files ';' block '.'
;

fixed_part :  fixed_part ';' record_section
|  record_section
;

compound_stmt :  SBEGIN statements END
;

expressions :  expressions ',' expression
|  expression
;

actual_params : 
|  '(' actuals_list ')'
;

formal_params :  '(' formal_p_sects ')'
;

newident :  IDENTIFIER
;

factor :  '(' expression ')'
|  unsigned_lit
|  NOT factor
|  set
|  variable
;

param_group :  newident_list ':' paramtype
;

case_list :  case_list_elem
|  case_list ';' case_list_elem
;

unsigned_lit :  STRING
|  unsigned_num
;

assignment :  variable BECOMES expression
;

function_form :  formal_params ':' ident
;

procedure_call :  ident actual_params
;

paramtype :  ARRAY '[' index_specs ']' OF paramtype
|  ident
|  PACKED ARRAY '[' index_spec ']' OF ident
;

opt_declarations :  declarations
| 
;

field_list :  fixed_part ';' variant_part
|  fixed_part
|  variant_part
;

statement_part :  compound_stmt
;

case_list_elem : 
|  case_label_list ':' statement
;

var_dcl_part :  VAR variable_dcls ';'
;

unsigned_num :  UNSIGNED_INT
;

type :  PACKED struct_type
|  struct_type
|  simple_type
;

declarations :  declaration
|  declarations declaration
;

record_var :  variable
;

proc_or_func :  func_heading ';' body ';'
|  proc_heading ';' body ';'
;

formal_p_sects :  formal_p_sect
|  formal_p_sects ';' formal_p_sect
;

newident_list :  new_id_list
;

variable_dcls :  variable_dcls ';' variable_dcl
|  variable_dcl
;

const_def :  newident '=' constant
;

simple_type :  '(' newident_list ')'
|  constant DOTDOT constant
|  ident
;

case_label :  constant
;

colon_things :  ':' expression
|  ':' expression ':' expression
;

constant :  '-' ident
|  '-' unsigned_num
|  '+' ident
|  ident
|  STRING
|  '+' unsigned_num
|  unsigned_num
;

new_id_list :  new_id_list ',' newident
|  newident
;

member_list :  members
;

body :  IDENTIFIER
|  block
;

actual_param :  expression
|  expression colon_things
;

member :  expression DOTDOT expression
|  expression
;

members :  member
|  members ',' member
;

proc_heading :  PROCEDURE newident formal_params
;

expression :  simple_expr
|  simple_expr relational_op simple_expr
;

index_t_list :  index_t_list ',' simple_type
|  simple_type
;

tag_field :  ident
|  newident ':' ident
;

label_dcl_part :  LABEL labels ';'
;

variants :  variant
|  variants ';' variant
;

block :  opt_declarations statement_part
;

variable_dcl :  newident_list ':' type
;

type_dcl_part :  TYPE type_defs ';'
;

index_spec :  newident DOTDOT newident ':' ident
;

func_heading :  FUNCTION newident function_form
;

variant :  case_label_list ':' '(' field_list ')'
| 
;

rec_var_list :  record_var
|  rec_var_list ',' record_var
;

formal_p_sect :  proc_heading
|  param_group
|  func_heading
|  VAR param_group
;

const_dcl_part :  CONST const_defs ';'
;

type_def :  newident '=' type
;

labels :  labels ',' label
|  label
;

variable :  variable '^'
|  variable '.' ident
|  ident actual_params
|  variable '[' expressions ']'
;

declaration :  const_dcl_part
|  type_dcl_part
|  proc_dcl_part
|  label_dcl_part
|  var_dcl_part
;

relational_op :  '='
;

case_label_list :  case_label
|  case_label_list ',' case_label
;

actuals_list :  actuals_list ',' actual_param
|  actual_param
;

record_section :  newident_list ':' type
| 
;

label :  UNSIGNED_INT
;

type_defs :  type_def
|  type_defs ';' type_def
;

ident :  IDENTIFIER
;

statement :  compound_stmt
|  WITH rec_var_list DO statement
|  IF expression THEN statement ELSE statement
|  FOR ident BECOMES expression Term_direction expression DO statement
|  WHILE expression DO statement
|  procedure_call
|  CASE expression OF case_list END
|  REPEAT statements UNTIL expression
|  label ':' statement
| 
|  assignment
;

proc_dcl_part :  proc_or_func
;

variant_part :  CASE tag_field OF variants
;

term :  factor
|  NOT term
|  term Term_mult_op factor
;

index_specs :  index_specs ';' index_spec
|  index_spec
;

const_defs :  const_defs ';' const_def
|  const_def
;

statements :  statements ';' statement
|  statement
;

struct_type :  SFILE OF type
|  ARRAY '[' index_t_list ']' OF type
|  RECORD field_list END
|  SET OF simple_type
;

simple_expr :  '+' term
|  term
|  '-' term
|  simple_expr Term_add_op term
;

set :  '[' member_list ']'
;
