%token DOTDOT
%token IF
%token THEN
%token Term_mult_op
%token REPEAT
%token Term_func_heading
%token Term_block
%token Term_opt_declarations
%token OF
%token SBEGIN
%token Term_external_files
%token WHILE
%token Term_program
%token Term_direction
%token Term_declarations
%token DO
%token Term_proc_or_func
%token Term_declaration
%token WITH
%token UNSIGNED_INT
%token UNTIL
%token Term_ident
%token END
%token Term_proc_dcl_part
%token ELSE
%token Term_body
%token NOT
%token CASE
%token BECOMES
%token Term_proc_heading
%token FOR
%token PROGRAM
%token Term_newident

%%

program :  Term_program
|  PROGRAM Term_newident Term_external_files ';' block '.'
;

compound_stmt :  SBEGIN statements END
;

actual_param :  expression
|  expression colon_things
;

expressions :  expressions ',' expression
|  expression
;

member :  expression DOTDOT expression
|  expression
;

members :  member
|  members ',' member
;

expression :  simple_expr
|  simple_expr relational_op simple_expr
;

actual_params :  '(' actuals_list ')'
;

factor :  '(' expression ')'
|  unsigned_lit
|  NOT factor
|  set
|  variable
;

case_list :  case_list_elem
|  case_list ';' case_list_elem
;

block :  Term_block
|  opt_declarations statement_part
;

unsigned_lit :  unsigned_num
;

assignment :  variable BECOMES expression
;

procedure_call :  Term_ident actual_params
;

opt_declarations :  Term_opt_declarations
|  declarations
;

rec_var_list :  record_var
|  rec_var_list ',' record_var
;

statement_part :  compound_stmt
;

case_list_elem : 
|  case_label_list ':' statement
;

add_op :  '<'
;

variable :  variable '^'
|  variable '.' Term_ident
|  Term_ident actual_params
|  variable '[' expressions ']'
;

unsigned_num :  UNSIGNED_INT
;

declaration :  Term_declaration
|  proc_dcl_part
;

declarations :  Term_declarations
|  declaration
|  declarations declaration
;

record_var :  variable
;

relational_op :  '<'
;

case_label_list :  case_label
;

actuals_list :  actuals_list ',' actual_param
|  actual_param
;

proc_or_func :  Term_proc_or_func
|  Term_func_heading ';' body ';'
|  Term_proc_heading ';' body ';'
;

label :  UNSIGNED_INT
;

statement :  compound_stmt
|  WITH rec_var_list DO statement
|  IF expression THEN statement ELSE statement
|  FOR Term_ident BECOMES expression Term_direction expression DO statement
|  WHILE expression DO statement
|  procedure_call
|  CASE expression OF case_list END
|  REPEAT statements UNTIL expression
|  label ':' statement
| 
|  assignment
;

proc_dcl_part :  Term_proc_dcl_part
|  proc_or_func
;

term :  factor
|  term Term_mult_op factor
;

case_label :  constant
;

colon_things :  ':' expression
|  ':' expression ':' expression
;

constant :  unsigned_num
;

statements :  statements ';' statement
|  statement
;

member_list :  members
;

simple_expr :  '+' term
|  term
|  '-' term
|  simple_expr add_op term
;

set :  '[' member_list ']'
;

body :  Term_body
|  block
;
